@@ -101,7 +101,8 @@ vardchanges <- function(Y, H, PSU, w_final,
               if (min(q %in% names(datasetX)) == 1) q <- datasetX[, q,  with = FALSE] } 
      }
 
-  equal_dataset <- all.equal(dataset, datasetX) & !is.null(X)
+  equal_dataset <- identical(dataset, datasetX) & !is.null(dataset) & !is.null(X)
+  dataset <- datasetX <- NULL
   if (equal_dataset) X_ID_level1 <- ID_level1
   if (equal_dataset) countryX <- country
 
@@ -109,39 +110,37 @@ vardchanges <- function(Y, H, PSU, w_final,
   Y <- data.table(Y, check.names = TRUE)
   n <- nrow(Y)
   m <- ncol(Y)
+  if (anyNA(Y)) stop("'Y' has missing values")
   if (!all(sapply(Y, is.numeric))) stop("'Y' must be numeric")
-  if (any(is.na(Y))) stop("'Y' has missing values")
-  if (is.null(names(Y))) stop("'Y' must have column names")
   if (any(grepl("__", names(Y)))) stop("'Y' is not allowed column names with '__'")
 
   # H
   H <- data.table(H)
   if (nrow(H) != n) stop("'H' length must be equal with 'Y' row count")
   if (ncol(H) != 1) stop("'H' must be 1 column data.frame, matrix, data.table")
-  if (is.null(names(H))) stop("'H' must be colname")
   H[, (names(H)) := lapply(.SD, as.character)]
-  if (any(is.na(H))) stop("'H' has missing values")
+  if (anyNA(H)) stop("'H' has missing values")
 
   # PSU
   PSU <- data.table(PSU)
   if (nrow(PSU) !=  n) stop("'PSU' length must be equal with 'Y' row count")
   if (ncol(PSU) !=  1) stop("'PSU' has more than 1 column")
   PSU[, (names(PSU)) := lapply(.SD, as.character)]
-  if (any(is.na(PSU))) stop("'PSU' has missing values")
+  if (anyNA(PSU)) stop("'PSU' has missing values")
   
   # w_final
   w_final <- data.frame(w_final)
+  if (anyNA(w_final)) stop("'w_final' has missing values") 
   if (nrow(w_final) !=  n) stop("'w_final' must be equal with 'Y' row count")
   if (ncol(w_final) !=  1) stop("'w_final' must be a vector or 1 column data.frame, matrix, data.table")
   w_final <- w_final[, 1]
   if (!is.numeric(w_final)) stop("'w_final' must be numeric")
-  if (any(is.na(w_final))) stop("'w_final' has missing values") 
   
   # ID_level1
   if (is.null(ID_level1)) stop("'ID_level1' must be defined")
   ID_level1 <- data.table(ID_level1)
   ID_level1[, (names(ID_level1)) := lapply(.SD, as.character)]
-  if (any(is.na(ID_level1))) stop("'ID_level1' has missing values")
+  if (anyNA(ID_level1)) stop("'ID_level1' has missing values")
   if (ncol(ID_level1) != 1) stop("'ID_level1' must be 1 column data.frame, matrix, data.table")
   if (nrow(ID_level1) != n) stop("'ID_level1' must be the same length as 'Y'")
   if (names(ID_level1) == names(PSU)) setnames(PSU, names(PSU), paste0(names(PSU), "_PSU")) 
@@ -149,7 +148,7 @@ vardchanges <- function(Y, H, PSU, w_final,
   # ID_level2
   ID_level2 <- data.table(ID_level2)
   ID_level2[, (names(ID_level2)) := lapply(.SD, as.character)]
-  if (any(is.na(ID_level2))) stop("'ID_level2' has missing values")
+  if (anyNA(ID_level2)) stop("'ID_level2' has missing values")
   if (nrow(ID_level2) != n) stop("'ID_level2' length must be equal with 'Y' row count")
   if (ncol(ID_level2) != 1) stop("'ID_level2' must be 1 column data.frame, matrix, data.table")
   if (names(ID_level2) == names(ID_level1)) setnames(ID_level2, names(ID_level2), paste0(names(ID_level2), "_id"))
@@ -158,7 +157,7 @@ vardchanges <- function(Y, H, PSU, w_final,
   if (!is.null(country)){
         country <- data.table(country)
         country[, (names(country)) := lapply(.SD, as.character)]
-        if (any(is.na(country))) stop("'country' has missing values")
+        if (anyNA(country)) stop("'country' has missing values")
         if (names(country) == "percoun") stop("'country' must be different name")
         if (nrow(country) != n) stop("'country' length must be equal with 'Y' row count")
         if (ncol(country) != 1) stop("'country' has more than 1 column")
@@ -167,7 +166,7 @@ vardchanges <- function(Y, H, PSU, w_final,
   # period
   period <- data.table(period, check.names = TRUE)
   period[, (names(period)) := lapply(.SD, as.character)]
-  if (any(is.na(period))) stop("'period' has missing values")
+  if (anyNA(period)) stop("'period' has missing values")
   if (names(period) == "percoun") stop("'period' must be different name")
   if (nrow(period) != n) stop("'period' length must be equal with 'Y' row count")
 
@@ -178,19 +177,17 @@ vardchanges <- function(Y, H, PSU, w_final,
            stop("'Dom' are duplicate column names: ", 
                  paste(names(Dom)[duplicated(names(Dom))], collapse = ","))
     if (nrow(Dom) != n) stop("'Dom' and 'Y' must be equal row count")
-    if (is.null(names(Dom))) stop("'Dom' must have column names")
     Dom[, (names(Dom)) := lapply(.SD, as.character)]
-    if (any(is.na(Dom))) stop("'Dom' has missing values")
+    if (anyNA(Dom)) stop("'Dom' has missing values")
     if (any(grepl("__", names(Dom)))) stop("'Dom' is not allowed column names with '__'")
   }
   
   namesZ <- NULL
   if (!is.null(Z)) {
     Z <- data.table(Z, check.names=TRUE)
+    if (anyNA(Z)) stop("'Z' has missing values")
     if (nrow(Z) !=  n) stop("'Z' and 'Y' must be equal row count")
     if (ncol(Z) !=  m) stop("'Z' and 'Y' must be equal column count")
-    if (any(is.na(Z))) stop("'Z' has missing values")
-    if (is.null(names(Z))) stop("'Z' must have column names")
     if (any(grepl("__", names(Z)))) stop("'Z' is not allowed column names with '__'")
     namesZ <- names(Z)
   }
@@ -200,11 +197,11 @@ vardchanges <- function(Y, H, PSU, w_final,
    if (ncol(period1) != 1) stop("'period1' must be 1 column")
    setnames(period1, names(period1), names(period))
    period1[, (names(period1)) := lapply(.SD, as.character)]
-   if (any(is.na(period1))) stop("'period1' has missing values")
+   if (anyNA(period1)) stop("'period1' has missing values")
    periods <- copy(period)
    periods[, periods := 1]
-   if (any(is.na(merge(period1, periods, all.x = TRUE,
-                        by = names(period), allow.cartesian = TRUE))))
+   if (anyNA(merge(period1, periods, all.x = TRUE,
+                        by = names(period), allow.cartesian = TRUE)))
               stop("'period1' row must be exist in 'period'")
 
 
@@ -213,14 +210,15 @@ vardchanges <- function(Y, H, PSU, w_final,
    if (ncol(period2) != 1) stop("'period2' must be 1 column")
    setnames(period2, names(period2), names(period))
    period2[, (names(period2)) := lapply(.SD, as.character)]
-   if (any(is.na(period2))) stop("'period2' has missing values")
-   if (any(is.na(merge(period2, periods, all.x = TRUE,
-                         by = names(period), allow.cartesian = TRUE))))
+   if (anyNA(period2)) stop("'period2' has missing values")
+   if (anyNA(merge(period2, periods, all.x = TRUE,
+                         by = names(period), allow.cartesian = TRUE)))
              stop("'period2' row must be exist in 'period'")
 
   if (!is.null(X)) {
     X <- data.table(X, check.names = TRUE)
     if (!all(sapply(X, is.numeric))) stop("'X' must be numeric values")
+    if (anyNA(X)) stop("'X' has missing values")
   }
 
   # countryX
@@ -230,7 +228,7 @@ vardchanges <- function(Y, H, PSU, w_final,
         if (nrow(countryX) != nrow(X)) stop("'countryX' length must be equal with 'X' row count")
         if (ncol(countryX) != 1) stop("'countryX' has more than 1 column")
         countryX[, (names(countryX)) := lapply(.SD, as.character)]
-        if (any(is.na(countryX))) stop("'countryX' has missing values")
+        if (anyNA(countryX)) stop("'countryX' has missing values")
         if (names(countryX) != names(country)) stop("'countryX' must be equal with 'country' names")
         countrX <- countryX[, .N, keyby = names(countryX)][, N := NULL]
         countr <- country[, .N, keyby = names(country)][, N := NULL]
@@ -243,7 +241,7 @@ vardchanges <- function(Y, H, PSU, w_final,
      if (is.null(periodX)) stop("'periodX' must be defined")
      periodX <- data.table(periodX)
      periodX[, (names(periodX)) := lapply(.SD, as.character)]
-     if (any(is.na(periodX))) stop("'periodX' has missing values")
+     if (anyNA(periodX)) stop("'periodX' has missing values")
      if (any(duplicated(names(periodX)))) 
                  stop("'periodX' are duplicate column names: ", 
                         paste(names(periodX)[duplicated(names(periodX))], collapse = ","))
@@ -260,34 +258,32 @@ vardchanges <- function(Y, H, PSU, w_final,
      if (any(peri != periX) & !is.null(country)) stop("'unique(country, period)' and 'unique(countryX, periodX)' records have different")
    } 
 
-
   # X_ID_level1
   if (!is.null(X)) {
-    X_ID_level1 <- data.table(X_ID_level1)
-    X_ID_level1[, (names(X_ID_level1)) := lapply(.SD, as.character)]
-    if (any(is.na(X_ID_level1))) stop("'X_ID_level1' has missing values")
-    if (nrow(X) != nrow(X_ID_level1)) stop("'X' and 'X_ID_level1' have different row count")
-    if (ncol(X_ID_level1) != 1) stop("'X_ID_level1' must be 1 column data.frame, matrix, data.table")
- 
-    ID_level1h <- copy(ID_level1)
-    if (!is.null(countryX)) { X_ID_level1 <- data.table(countryX, X_ID_level1)
-                              ID_level1h <- data.table(country, ID_level1h)}
-    X_ID_level1 <- data.table(periodX, X_ID_level1)
-    ID_level1h <- data.table(period, ID_level1h)
-    ID_level1h <- ID_level1h[, .N, by = names(ID_level1h)][, N := NULL]
-    if (nrow(X_ID_level1[, .N, by = names(X_ID_level1)][N > 1]) > 0) stop("'X_ID_level1' have duplicates")
-    setkeyv(X_ID_level1, names(X_ID_level1))
-    setkeyv(ID_level1h, names(ID_level1h))
-    nperIDh <- names(ID_level1h)
-    if (any(nperIDh != names(X_ID_level1))) stop("'X_ID_level1' and 'ID_level1' must be equal names")
-    if (ID_level1h[, class(get(nperIDh))] != X_ID_level1[, class(get(nperIDh))])  stop("Class for 'X_ID_level1' and class for 'ID_level1' must be equal ")
+     if (is.null(X_ID_level1)) stop("'X_ID_level1' must be defined")
+     X_ID_level1 <- data.table(X_ID_level1)
+     X_ID_level1[, (names(X_ID_level1)) := lapply(.SD, as.character)]
+     if (anyNA(X_ID_level1)) stop("'X_ID_level1' has missing values")
+     if (nrow(X) != nrow(X_ID_level1)) stop("'X' and 'X_ID_level1' have different row count")
+     if (ncol(X_ID_level1) != 1) stop("'X_ID_level1' must be 1 column data.frame, matrix, data.table")
+     if (any(names(X_ID_level1) != names(ID_level1))) stop("'X_ID_level1' and 'ID_level1' must be equal names")
 
-    if (!is.null(country)) {
-          if (nrow(ID_level1h) != nrow(X_ID_level1)) stop("'unique(countryX, periodX, X_ID_level1)' and 'unique(country, period, ID_level1)' have different row count")
-          if (any(ID_level1h != X_ID_level1)) stop("''unique(countryX, periodX, X_ID_level1)' and 'unique(country, period, ID_level1)' records have different")
+     ID_level1h <- data.table(period, ID_level1)
+     X_ID_level1h <- data.table(periodX, X_ID_level1)
+     if (!is.null(countryX)) { X_ID_level1 <- data.table(countryX, X_ID_level1h)
+                               ID_level1h <- data.table(country, ID_level1h)}
+     ID_level1h <- ID_level1h[, .N, by = names(ID_level1h)][, N := NULL]
+     if (nrow(X_ID_level1h[, .N, by = names(X_ID_level1h)][N > 1]) > 0) stop("'X_ID_level1' have duplicates")
+     setkeyv(X_ID_level1h, names(X_ID_level1h))
+     setkeyv(ID_level1h, names(ID_level1h))
+
+     if (!is.null(country)) {
+           if (nrow(ID_level1h) != nrow(X_ID_level1h)) stop("'unique(countryX, periodX, X_ID_level1)' and 'unique(country, period, ID_level1)' have different row count")
+           if (any(ID_level1h != X_ID_level1h)) stop("''unique(countryX, periodX, X_ID_level1)' and 'unique(country, period, ID_level1)' records have different")
        } else {
-          if (nrow(ID_level1h) != nrow(X_ID_level1)) stop("'unique(periodX, X_ID_level1)' and 'unique(period, ID_level1)' have different row count")
-          if (any(ID_level1h != X_ID_level1)) stop("''unique(periodX, X_ID_level1)' and 'unique(period, ID_level1)' records have different")  }
+           if (nrow(ID_level1h) != nrow(X_ID_level1h)) stop("'unique(periodX, X_ID_level1)' and 'unique(period, ID_level1)' have different row count")
+           if (any(ID_level1h != X_ID_level1h)) stop("''unique(periodX, X_ID_level1)' and 'unique(period, ID_level1)' records have different")  }
+     ID_level1h <- X_ID_level1h <- NULL
   }
 
   # ind_gr
@@ -297,32 +293,18 @@ vardchanges <- function(Y, H, PSU, w_final,
      if (nrow(ind_gr) != nrow(X)) stop("'ind_gr' length must be equal with 'X' row count")
      if (ncol(ind_gr) != 1) stop("'ind_gr' must be 1 column data.frame, matrix, data.table")
      ind_gr[, (names(ind_gr)) := lapply(.SD, as.character)]
-     if (any(is.na(ind_gr))) stop("'ind_gr' has missing values")
+     if (anyNA(ind_gr)) stop("'ind_gr' has missing values")
    }
 
-  # X
-  if (!is.null(X)) {
-       X1 <- data.table(X, check.names = TRUE)
-       nX1 <- names(X1)
-       ind_gr1 <- data.table(periodX, ind_gr, check.names = TRUE)
-       X2 <- data.table(ind_gr1, X1)
-       X1 <- X2[, .N, keyby = names(ind_gr1)][[ncol(ind_gr1) + 1]]
-       X2 <- X2[, lapply(.SD, function(x) sum(!is.na(x))), keyby = names(ind_gr1), .SDcols = nX1]
-       X2 <- X2[, !(names(X2) %in% names(ind_gr1)), with = FALSE]
-
-       if (!all(X2 == 0 | X1 == X2)) stop("X has missing values")
-       ind_gr1 <- nX1 <- nX2 <- X1 <- X2 <- NULL
-    }
-
   # g
   if (!is.null(X)) {
     if (is.null(class(g)) | all(class(g) == "function")) stop("'g' must be numeric")
     g <- data.frame(g)
+    if (anyNA(g)) stop("'g' has missing values")
     if (nrow(g) != nrow(X)) stop("'g' length must be equal with 'X' row count")
     if (ncol(g) != 1) stop("'g' must be 1 column data.frame, matrix, data.table")
     g <- g[, 1]
     if (!is.numeric(g)) stop("'g' must be numeric")
-    if (any(is.na(g))) stop("'g' has missing values")
     if (any(g == 0)) stop("'g' value can not be 0")
    }
     
@@ -331,11 +313,11 @@ vardchanges <- function(Y, H, PSU, w_final,
     if (is.null(q))  q <- rep(1, nrow(X))
     if (is.null(class(q)) | all(class(q) == "function")) stop("'q' must be numeric")
     q <- data.frame(q)
+    if (anyNA(q)) stop("'q' has missing values")
     if (nrow(q) != nrow(X)) stop("'q' length must be equal with 'X' row count")
     if (ncol(q) != 1) stop("'q' must be 1 column data.frame, matrix, data.table")
     q <- q[, 1]
     if (!is.numeric(q)) stop("'q' must be numeric")
-    if (any(is.na(q))) stop("'q' has missing values")
     if (any(is.infinite(q))) stop("'q' value can not be infinite")
   }
 
@@ -343,13 +325,13 @@ vardchanges <- function(Y, H, PSU, w_final,
                      ID_level1 = ID_level1, ID_level2 = ID_level2,
                      Dom = Dom, Z = Z, country = country,
                      period = period, dataset = NULL,
-                     countryX = countryX, periodX = periodX,
+                     X = X, countryX = countryX, periodX = periodX,
                      X_ID_level1 = X_ID_level1, ind_gr = ind_gr,
                      g = g, q = q, datasetX = NULL,
                      linratio = linratio,
                      percentratio = percentratio,
                      use.estVar = use.estVar,
-                     ID_level1_max = TRUE,
+                     ID_level1_max = is.null(X),
                      outp_res = outp_res,
                      withperiod = TRUE,
                      netchanges = TRUE, 
@@ -357,6 +339,9 @@ vardchanges <- function(Y, H, PSU, w_final,
   res_out <- datas$res_out
   crossectional_results <- datas$results
 
+  countryX <- periodX <- X_ID_level1 <- NULL
+  X_ID_level1 <- ind_gr <- g  <- q  <- NULL
+
   if (!is.null(Dom)) { 
         Y1 <- names(domain(Y, Dom))
         if (!is.null(Z)) Z1 <- names(domain(Z, Dom))
