@@ -31,7 +31,7 @@ vardcros <- function(Y, H, PSU, w_final,
   if (length(use.estVar) != 1 | !any(is.logical(use.estVar))) stop("'use.estVar' must be logical")
   if (length(ID_level1_max) != 1 | !any(is.logical(ID_level1_max))) stop("'ID_level1_max' must be logical")
   if (length(outp_res) != 1 | !any(is.logical(outp_res))) stop("'outp_res' must be logical")
- if (all(ID_level1_max, !is.null(X))) stop("'ID_level1_max' must be ", !ID_level1_max, "!")
+  if (all(ID_level1_max, !is.null(X))) stop("'ID_level1_max' must be ", !ID_level1_max, "!")
 
   if(length(confidence) != 1 | any(!is.numeric(confidence) |  confidence < 0 | confidence > 1)) {
           stop("'confidence' must be a numeric value in [0, 1]")  }
@@ -111,7 +111,8 @@ vardcros <- function(Y, H, PSU, w_final,
               if (min(q %in% names(datasetX)) != 1) stop("'q' does not exist in 'datasetX'!") 
               if (min(q %in% names(datasetX)) == 1) q <- datasetX[, q,  with = FALSE] } 
      }
-  equal_dataset <- all.equal(dataset, datasetX) & !is.null(X)
+
+  equal_dataset <- identical(dataset, datasetX) & !is.null(datasetX) & !is.null(X)
   if (equal_dataset) X_ID_level1 <- ID_level1
   if (equal_dataset) countryX <- country
 
@@ -122,24 +123,23 @@ vardcros <- function(Y, H, PSU, w_final,
   n <- nrow(Y)
   m <- ncol(Y)
   if (!all(sapply(Y, is.numeric))) stop("'Y' must be numeric values")
-  if (any(is.na(Y))) stop("'Y' has missing values")
+  if (anyNA(Y)) stop("'Y' has missing values")
   if (is.null(names(Y))) stop("'Y' must have column names")
   
   # H
   H <- data.table(H)
   if (nrow(H) != n) stop("'H' length must be equal with 'Y' row count")
   if (ncol(H) != 1) stop("'H' must be 1 column data.frame, matrix, data.table")
-  if (is.null(names(H))) stop("'H' must have column names")
   if (names(H) == "dataH_stratas") stop("'H' must have different column name")
   H[, (names(H)) := lapply(.SD, as.character)]
-  if (any(is.na(H))) stop("'H' has missing values")
+  if (anyNA(H)) stop("'H' has missing values")
 
   # PSU
   PSU <- data.table(PSU)
   if (nrow(PSU) != n) stop("'PSU' length must be equal with 'Y' row count")
   if (ncol(PSU) != 1) stop("'PSU' has more than 1 column")
   PSU[, (names(PSU)) := lapply(.SD, as.character)]
-  if (any(is.na(PSU))) stop("'PSU' has missing values")
+  if (anyNA(PSU)) stop("'PSU' has missing values")
   
   # w_final 
   w_final <- data.frame(w_final)
@@ -147,13 +147,13 @@ vardcros <- function(Y, H, PSU, w_final,
   if (ncol(w_final) != 1) stop("'w_final' must be a vector or 1 column data.frame, matrix, data.table")
   w_final <- w_final[, 1]
   if (!is.numeric(w_final)) stop("'w_final' must be numeric values")
-  if (any(is.na(w_final))) stop("'w_final' has missing values") 
+  if (anyNA(w_final)) stop("'w_final' has missing values") 
   
   # ID_level1
   if (is.null(ID_level1)) stop("'ID_level1' must be defined")
   ID_level1 <- data.table(ID_level1)
   ID_level1[, (names(ID_level1)) := lapply(.SD, as.character)]
-  if (any(is.na(ID_level1))) stop("'ID_level1' has missing values")
+  if (anyNA(ID_level1)) stop("'ID_level1' has missing values")
   if (ncol(ID_level1) != 1) stop("'ID_level1' must be 1 column data.frame, matrix, data.table")
   if (nrow(ID_level1) != n) stop("'ID_level1' must be the same length as 'Y'")
   if (names(ID_level1) == names(PSU)) setnames(PSU, names(PSU), paste0(names(PSU), "_PSU")) 
@@ -161,7 +161,7 @@ vardcros <- function(Y, H, PSU, w_final,
   # ID_level2
   ID_level2 <- data.table(ID_level2)
   ID_level2[, (names(ID_level2)) := lapply(.SD, as.character)]
-  if (any(is.na(ID_level2))) stop("'ID_level2' has missing values")
+  if (anyNA(ID_level2)) stop("'ID_level2' has missing values")
   if (nrow(ID_level2) != n) stop("'ID_level2' length must be equal with 'Y' row count")
   if (ncol(ID_level2) != 1) stop("'ID_level2' must be 1 column data.frame, matrix, data.table")
   if (names(ID_level2) == names(ID_level1)) setnames(ID_level2, names(ID_level2), paste0(names(ID_level2), "_id"))
@@ -170,7 +170,7 @@ vardcros <- function(Y, H, PSU, w_final,
   if (!is.null(country)){
         country <- data.table(country)
         country[, (names(country)) := lapply(.SD, as.character)]
-        if (any(is.na(country))) stop("'country' has missing values")
+        if (anyNA(country)) stop("'country' has missing values")
         if (names(country) == "percoun") stop("'country' must be different name")
         if (nrow(country) != n) stop("'country' length must be equal with 'Y' row count")
         if (ncol(country) != 1) stop("'country' has more than 1 column")
@@ -180,7 +180,7 @@ vardcros <- function(Y, H, PSU, w_final,
   if (withperiod) {
         period <- data.table(period)
         period[, (names(period)) := lapply(.SD, as.character)]
-        if (any(is.na(period))) stop("'period' has missing values")
+        if (anyNA(period)) stop("'period' has missing values")
         if (names(period) == "percoun") stop("'period' must be different name")
         if (nrow(period) != n) stop("'period' length must be equal with 'Y' row count")
     } else if (!is.null(period)) stop("'period' must be NULL for those data")
@@ -193,10 +193,9 @@ vardcros <- function(Y, H, PSU, w_final,
            stop("'Dom' are duplicate column names: ", 
                  paste(names(Dom)[duplicated(names(Dom))], collapse = ","))
     if (nrow(Dom) != n) stop("'Dom' and 'Y' must be equal row count")
-    if (is.null(names(Dom))) stop("'Dom' must have column names")
     namesDom <- names(Dom)
     Dom[, (namesDom) := lapply(.SD, as.character)]
-    if (any(is.na(Dom))) stop("'Dom' has missing values")
+    if (anyNA(Dom)) stop("'Dom' has missing values")
     if (any(grepl("__", namesDom))) stop("'Dom' is not allowed column names with '__'")
     Dom_agg <- Dom[,.N, keyby = namesDom][, N := NULL]
     Dom_agg1 <- Dom_agg[, lapply(namesDom, function(x) make.names(paste0(x, ".", get(x))))]
@@ -207,17 +206,17 @@ vardcros <- function(Y, H, PSU, w_final,
   # Z
   if (!is.null(Z)) {
     Z <- data.table(Z, check.names = TRUE)
+    if (anyNA(Z)) stop("'Z' has missing values")
     if (!all(sapply(Z, is.numeric))) stop("'Z' must be numeric values")
     if (nrow(Z) != n) stop("'Z' and 'Y' must be equal row count")
     if (ncol(Z) != m) stop("'Z' and 'Y' must be equal column count")
-    if (any(is.na(Z))) stop("'Z' has missing values")
-    if (is.null(names(Z))) stop("'Z' must have column names")
     if (any(grepl("__", names(Z)))) stop("'Z' is not allowed column names with '__'")
   }
       
   if (!is.null(X)) {
     X <- data.table(X, check.names = TRUE)
     if (!all(sapply(X, is.numeric))) stop("'X' must be numeric values")
+    if (anyNA(X)) stop("'X' has missing values")
   }
 
   # countryX
@@ -227,7 +226,7 @@ vardcros <- function(Y, H, PSU, w_final,
         if (nrow(countryX) != nrow(X)) stop("'countryX' length must be equal with 'X' row count")
         if (ncol(countryX) != 1) stop("'countryX' has more than 1 column")
         countryX[, (names(countryX)) := lapply(.SD, as.character)]
-        if (any(is.na(countryX))) stop("'countryX' has missing values")
+        if (anyNA(countryX)) stop("'countryX' has missing values")
         if (names(countryX) != names(country)) stop("'countryX' must be equal with 'country' names")
         countrX <- countryX[, .N, keyby = names(countryX)][, N := NULL]
         countr <- country[, .N, keyby = names(country)][, N := NULL]
@@ -240,7 +239,7 @@ vardcros <- function(Y, H, PSU, w_final,
      if(!is.null(periodX)) {
         periodX <- data.table(periodX)
         periodX[, (names(periodX)) := lapply(.SD, as.character)]
-        if (any(is.na(periodX))) stop("'periodX' has missing values")
+        if (anyNA(periodX)) stop("'periodX' has missing values")
         if (any(duplicated(names(periodX)))) 
                     stop("'periodX' are duplicate column names: ", 
                          paste(names(periodX)[duplicated(names(periodX))], collapse = ","))
@@ -260,41 +259,44 @@ vardcros <- function(Y, H, PSU, w_final,
 
 
   # X_ID_level1
-  if (!is.null(X)) {
-    X_ID_level1 <- data.table(X_ID_level1)
-    X_ID_level1[, (names(X_ID_level1)) := lapply(.SD, as.character)]
-    if (any(is.na(X_ID_level1))) stop("'X_ID_level1' has missing values")
-    if (nrow(X) != nrow(X_ID_level1)) stop("'X' and 'X_ID_level1' have different row count")
-    if (ncol(X_ID_level1) != 1) stop("'X_ID_level1' must be 1 column data.frame, matrix, data.table")
+    if (!is.null(X)) {
+      if (is.null(X_ID_level1)) stop("'X_ID_level1' must be defined")
+      X_ID_level1 <- data.table(X_ID_level1)
+      X_ID_level1[, (names(X_ID_level1)) := lapply(.SD, as.character)]
+      if (anyNA(X_ID_level1)) stop("'X_ID_level1' has missing values")
+      if (nrow(X) != nrow(X_ID_level1)) stop("'X' and 'X_ID_level1' have different row count")
+      if (ncol(X_ID_level1) != 1) stop("'X_ID_level1' must be 1 column data.frame, matrix, data.table")
+      if (any(names(X_ID_level1) != names(ID_level1))) stop("'X_ID_level1' and 'ID_level1' must be equal names")
+      
+      ID_level1h <- copy(ID_level1)
+      X_ID_level1h <- copy(X_ID_level1)
+      if (!is.null(countryX)) {X_ID_level1h <- data.table(countryX, X_ID_level1h)
+                               ID_level1h <- data.table(country, ID_level1h)}
+      if (!is.null(periodX)) {X_ID_level1h <- data.table(periodX, X_ID_level1h)
+                              ID_level1h <- data.table(period, ID_level1h)}
 
-    ID_level1h <- copy(ID_level1)
-    if (!is.null(countryX)) {X_ID_level1 <- data.table(countryX, X_ID_level1)
-                             ID_level1h <- data.table(country, ID_level1h)}
-    if (!is.null(periodX)) {X_ID_level1 <- data.table(periodX, X_ID_level1)
-                            ID_level1h <- data.table(period, ID_level1h)}
-    ID_level1h <- ID_level1h[, .N, by = names(ID_level1h)][, N := NULL]
-    if (nrow(X_ID_level1[,.N, by = names(X_ID_level1)][N > 1]) > 0) stop("'X_ID_level1' have duplicates")
-    setkeyv(X_ID_level1, names(X_ID_level1))
-    setkeyv(ID_level1h, names(ID_level1h))
-    nperIDh <- names(ID_level1h)
-    if (any(nperIDh != names(X_ID_level1))) stop("'X_ID_level1' and 'ID_level1' must be equal  names")
-    if (ID_level1h[, class(get(nperIDh))] != X_ID_level1[, class(get(nperIDh))])  stop("Class for 'X_ID_level1' and class for 'ID_level1' must be equal ")
+      ID_level1h <- ID_level1h[, .N, by = names(ID_level1h)][, N := NULL]
+      if (nrow(X_ID_level1h[,.N, by = names(X_ID_level1h)][N > 1]) > 0) stop("'X_ID_level1' have duplicates")  
 
-    if (!is.null(period)) {
-        if (!is.null(country)) {
-             if (nrow(ID_level1h) != nrow(X_ID_level1)) stop("'unique(countryX, periodX, X_ID_level1)' and 'unique(country, period, ID_level1)' have different row count")
-             if (any(ID_level1h != X_ID_level1)) stop("''unique(countryX, periodX, X_ID_level1)' and 'unique(country, period, ID_level1)' records have different")
+      setkeyv(X_ID_level1h, names(X_ID_level1h))
+      setkeyv(ID_level1h, names(ID_level1h))
+      if (!is.null(period)) {
+          if (!is.null(country)) {
+                 if (nrow(ID_level1h) != nrow(X_ID_level1h)) stop("'unique(countryX, periodX, X_ID_level1)' and 'unique(country, period, ID_level1)' have different row count")
+                 if (any(ID_level1h != X_ID_level1h)) stop("''unique(countryX, periodX, X_ID_level1)' and 'unique(country, period, ID_level1)' records have different")
+              } else {
+                 if (nrow(ID_level1h) != nrow(X_ID_level1h)) stop("'unique(periodX, X_ID_level1)' and 'unique(period, ID_level1)' have different row count")
+                 if (any(ID_level1h != X_ID_level1h)) stop("''unique(periodX, X_ID_level1)' and 'unique(period, ID_level1)' records have different")  }
+        } else {
+          if (!is.null(country)) {
+               if (nrow(ID_level1h) != nrow(X_ID_level1h)) stop("'unique(countryX, X_ID_level1)' and 'unique(country, ID_level1)' have different row count")
+               if (any(ID_level1h != X_ID_level1h)) stop("''unique(countryX, X_ID_level1)' and 'unique(country, ID_level1)' records have different")
             } else {
-                if (nrow(ID_level1h) != nrow(X_ID_level1)) stop("'unique(periodX, X_ID_level1)' and 'unique(period, ID_level1)' have different row count")
-                if (any(ID_level1h != X_ID_level1)) stop("''unique(periodX, X_ID_level1)' and 'unique(period, ID_level1)' records have different")  }
-      } else {
-        if (!is.null(country)) {
-             if (nrow(ID_level1h) != nrow(X_ID_level1)) stop("'unique(countryX, X_ID_level1)' and 'unique(country, ID_level1)' have different row count")
-             if (any(ID_level1h != X_ID_level1)) stop("''unique(countryX, X_ID_level1)' and 'unique(country, ID_level1)' records have different")
-            } else {
-             if (nrow(ID_level1h) != nrow(X_ID_level1)) stop("'unique(X_ID_level1)' and 'unique(ID_level1)' have different row count")
-             if (any(ID_level1h != X_ID_level1)) stop("''unique(X_ID_level1)' and 'unique(ID_level1)' records have different") }
-   }}
+               if (nrow(ID_level1h) != nrow(X_ID_level1h)) stop("'unique(X_ID_level1)' and 'unique(ID_level1)' have different row count")
+               if (any(ID_level1h != X_ID_level1h)) stop("''unique(X_ID_level1)' and 'unique(ID_level1)' records have different") }
+      }
+      ID_level1h <- X_ID_level1h <- NULL
+    }
 
   # ind_gr
   if (!is.null(X)) {
@@ -303,33 +305,18 @@ vardcros <- function(Y, H, PSU, w_final,
      if (nrow(ind_gr) != nrow(X)) stop("'ind_gr' length must be equal with 'X' row count")
      if (ncol(ind_gr) != 1) stop("'ind_gr' must be 1 column data.frame, matrix, data.table")
      ind_gr[, (names(ind_gr)) := lapply(.SD, as.character)]
-     if (any(is.na(ind_gr))) stop("'ind_gr' has missing values")
+     if (anyNA(ind_gr)) stop("'ind_gr' has missing values")
    }
 
-  # X
-  if (!is.null(X)) {
-       X1 <- data.table(X, check.names = TRUE)
-       nX1 <- names(X1)
-       ind_gr1 <- copy(ind_gr) 
-       if (!is.null(periodX)) ind_gr1 <- data.table(periodX, ind_gr1, check.names = TRUE)
-       X2 <- data.table(ind_gr1, X1)
-       X1 <- X2[, .N, keyby = names(ind_gr1)][[ncol(ind_gr1) + 1]]
-       X2 <- X2[, lapply(.SD, function(x) sum(!is.na(x))), keyby = names(ind_gr1), .SDcols = nX1]
-       X2 <- X2[, !(names(X2) %in% names(ind_gr1)), with = FALSE]
-
-       if (!all(X2 == 0 | X1 == X2)) stop("X has missing values")
-       ind_gr1 <- nX1 <- nX2 <- X1 <- X2 <- NULL
-    }
-
   # g
   if (!is.null(X)) {
     if (is.null(class(g)) | all(class(g) == "function")) stop("'g' must be numeric")
     g <- data.frame(g)
+    if (anyNA(g)) stop("'g' has missing values")
     if (nrow(g) != nrow(X)) stop("'g' length must be equal with 'X' row count")
     if (ncol(g) != 1) stop("'g' must be 1 column data.frame, matrix, data.table")
     g <- g[, 1]
     if (!is.numeric(g)) stop("'g' must be numeric")
-    if (any(is.na(g))) stop("'g' has missing values")
     if (any(g == 0)) stop("'g' value can not be 0")
    }
     
@@ -338,11 +325,11 @@ vardcros <- function(Y, H, PSU, w_final,
     if (is.null(q))  q <- rep(1, nrow(X))
     if (is.null(class(q)) | all(class(q) == "function")) stop("'q' must be numeric")
     q <- data.frame(q)
+    if (anyNA(q)) stop("'q' has missing values")
     if (nrow(q) != nrow(X)) stop("'q' length must be equal with 'X' row count")
     if (ncol(q) != 1) stop("'q' must be 1 column data.frame, matrix, data.table")
     q <- q[, 1]
     if (!is.numeric(q)) stop("'q' must be numeric")
-    if (any(is.na(q))) stop("'q' has missing values")
     if (any(is.infinite(q))) stop("'q' value can not be infinite")
   }
   
@@ -354,9 +341,12 @@ vardcros <- function(Y, H, PSU, w_final,
   # Design weights
   if (!is.null(X)) {
              idh <- data.table(ID_level1)
-             if (!is.null(country)) idh <- data.table(country, idh)
-             if (!is.null(period)) idh <- data.table(period, idh)
-             idhx <- data.table(X_ID_level1, g)
+             idhx <- data.table(X_ID_level1)
+             if (!is.null(countryX)) {idh <- data.table(country, idh)
+                                      idhx <- data.table(countryX, idhx)}
+             if (!is.null(periodX)) {idh <- data.table(period, idh)
+                                     idhx <- data.table(periodX, idhx)}
+             idhx <- data.table(idhx, g)
              setnames(idhx, names(idhx)[c(1 : (ncol(idhx)-1))], names(idh))
              idg <- merge(idh, idhx,  by = names(idh), sort = FALSE)
              w_design <- w_final / idg[[ncol(idg)]]
@@ -469,25 +459,28 @@ vardcros <- function(Y, H, PSU, w_final,
     res_outp <- NULL
     if (!is.null(X)) {
          X0 <- data.table(X_ID_level1, ind_gr, q, g, X)
-         DT1 <- merge(DTc, X0, by = names(ID_level1h))
+         if (!is.null(countryX)) X0 <- data.table(countryX, X0)
+         if (!is.null(periodX)) X0 <- data.table(periodX, X0)
+         nos <- c(names(periodX), names(countryX), names(ID_level1))
+         DT1 <- merge(DTc, X0, by = nos)
 
          ind_gr <- DT1[, c(namesperc, names(ind_gr)), with = FALSE]
-         ind_period <- do.call("paste", c(as.list(ind_gr), sep="_"))
+         ind_period <- do.call("paste", c(as.list(ind_gr), sep = "_"))
      
          res <- lapply(split(DT1[, .I], ind_period), function(i)                  
-                        data.table(DT1[i, names(ID_level1h), with = FALSE],
+                        data.table(DT1[i, nos, with = FALSE],
                                    res <- residual_est(Y = DT1[i, namesY2, with = FALSE],
                                                        X = DT1[i, names(X), with = FALSE],
-                                                       weight = DT1[i, "w_design", with = FALSE],
-                                                       q = DT1[i, "q", with = FALSE])))
- 
+                                                       weight = DT1[i, w_design],
+                                                       q = DT1[i, q])))
          res <- rbindlist(res)
          setnames(res, namesY2, namesY2w)
-         DTc <- merge(DTc, res, by = names(ID_level1h)) 
+         DTc <- merge(DTc, res, by = nos) 
          if (outp_res) res_outp <- DTc[, c(names(ID_level1h), names_PSU, "w_final", namesY2w), with = FALSE]
      } else DTc[, (namesY2w) := .SD[, namesY2, with = FALSE]]
+  
 
-   DTc[, (namesY2w):=.SD[, namesY2, with = FALSE] * get("w_final")]
+   DTc[, (namesY2w) := .SD[, namesY2w, with = FALSE] * get("w_final")]
 
    #--------------------------------------------------------*
    # AGGREGATION AT PSU LEVEL ("ULTIMATE CLUSTER" APPROACH) |
