@@ -99,8 +99,8 @@ vardcrosannual <- function(Y, H, PSU, w_final,
               if (min(q %in% names(datasetX)) != 1) stop("'q' does not exist in 'datasetX'!") 
               if (min(q %in% names(datasetX)) == 1) q <- datasetX[, q,  with = FALSE] } 
      }
-
-  equal_dataset <- all.equal(dataset, datasetX) & !is.null(X)
+ 
+  equal_dataset <- identical(dataset, datasetX) & !is.null(datasetX) & !is.null(X)
   if (equal_dataset) X_ID_level1 <- ID_level1
   if (equal_dataset) countryX <- country 
 
@@ -108,39 +108,37 @@ vardcrosannual <- function(Y, H, PSU, w_final,
   Y <- data.table(Y, check.names = TRUE)
   n <- nrow(Y)
   m <- ncol(Y)
+  if (anyNA(Y)) stop("'Y' has missing values")
   if (!all(sapply(Y, is.numeric))) stop("'Y' must be numeric")
-  if (any(is.na(Y))) stop("'Y' has missing values")
-  if (is.null(names(Y))) stop("'Y' must have column names")
   if (any(grepl("__", names(Y)))) stop("'Y' is not allowed column names with '__'")
 
   # H
   H <- data.table(H)
   if (nrow(H) != n) stop("'H' length must be equal with 'Y' row count")
   if (ncol(H) != 1) stop("'H' must be 1 column data.frame, matrix, data.table")
-  if (is.null(names(H))) stop("'H' must be colname")
   H[, (names(H)) := lapply(.SD, as.character)]
-  if (any(is.na(H))) stop("'H' has missing values")
+  if (anyNA(H)) stop("'H' has missing values")
 
   # PSU
   PSU <- data.table(PSU)
   if (nrow(PSU) !=  n) stop("'PSU' length must be equal with 'Y' row count")
   if (ncol(PSU) !=  1) stop("'PSU' has more than 1 column")
-  PSU[, (names(PSU)):=lapply(.SD, as.character)]
-  if (any(is.na(PSU))) stop("'PSU' has missing values")
+  PSU[, (names(PSU)) := lapply(.SD, as.character)]
+  if (anyNA(PSU)) stop("'PSU' has missing values")
   
   # w_final
   w_final <- data.frame(w_final)
+  if (anyNA(w_final)) stop("'w_final' has missing values") 
   if (nrow(w_final) !=  n) stop("'w_final' must be equal with 'Y' row count")
   if (ncol(w_final) !=  1) stop("'w_final' must be a vector or 1 column data.frame, matrix, data.table")
   w_final <- w_final[, 1]
   if (!is.numeric(w_final)) stop("'w_final' must be numeric")
-  if (any(is.na(w_final))) stop("'w_final' has missing values") 
   
   # ID_level1
   if (is.null(ID_level1)) stop("'ID_level1' must be defined")
   ID_level1 <- data.table(ID_level1)
   ID_level1[, (names(ID_level1)) := lapply(.SD, as.character)]
-  if (any(is.na(ID_level1))) stop("'ID_level1' has missing values")
+  if (anyNA(ID_level1)) stop("'ID_level1' has missing values")
   if (ncol(ID_level1) != 1) stop("'ID_level1' must be 1 column data.frame, matrix, data.table")
   if (nrow(ID_level1) != n) stop("'ID_level1' must be the same length as 'Y'")
   if (names(ID_level1) == names(PSU)) setnames(PSU, names(PSU), paste0(names(PSU), "_PSU")) 
@@ -148,7 +146,7 @@ vardcrosannual <- function(Y, H, PSU, w_final,
   # ID_level2
   ID_level2 <- data.table(ID_level2)
   ID_level2[, (names(ID_level2)) := lapply(.SD, as.character)]
-  if (any(is.na(ID_level2))) stop("'ID_level2' has missing values")
+  if (anyNA(ID_level2)) stop("'ID_level2' has missing values")
   if (nrow(ID_level2) != n) stop("'ID_level2' length must be equal with 'Y' row count")
   if (ncol(ID_level2) != 1) stop("'ID_level2' must be 1 column data.frame, matrix, data.table")
   if (names(ID_level2) == names(ID_level1)) setnames(ID_level2, names(ID_level2), paste0(names(ID_level2), "_id"))
@@ -157,7 +155,7 @@ vardcrosannual <- function(Y, H, PSU, w_final,
   if (!is.null(country)){
         country <- data.table(country)
         country[, (names(country)) := lapply(.SD, as.character)]
-        if (any(is.na(country))) stop("'country' has missing values")
+        if (anyNA(country)) stop("'country' has missing values")
         if (names(country) == "percoun") stop("'country' must be different name")
         if (nrow(country) != n) stop("'country' length must be equal with 'Y' row count")
         if (ncol(country) != 1) stop("'country' has more than 1 column")
@@ -166,7 +164,7 @@ vardcrosannual <- function(Y, H, PSU, w_final,
   # years
   years <- data.table(years, check.names = TRUE)
   years[, (names(years)) := lapply(.SD, as.character)]
-  if (any(is.na(years))) stop("'years' has missing values")
+  if (anyNA(years)) stop("'years' has missing values")
   if (nrow(years) != n) stop("'years' length must be equal with 'Y' row count")
   if (ncol(years) != 1) stop("'years' must be 1 column")
   yearm <- names(years)
@@ -174,7 +172,7 @@ vardcrosannual <- function(Y, H, PSU, w_final,
   # subperiods
   subperiods <- data.table(subperiods, check.names = TRUE)
   subperiods[, (names(subperiods)) := lapply(.SD, as.character)]
-  if (any(is.na(subperiods))) stop("'subperiods' has missing values")
+  if (anyNA(subperiods)) stop("'subperiods' has missing values")
   if (nrow(subperiods) != n) stop("'subperiods' length must be equal with 'Y' row count")
   if (ncol(subperiods) != 1) stop("'subperiods' must be 1 column")
   subn <- data.table(years, subperiods)
@@ -188,31 +186,36 @@ vardcrosannual <- function(Y, H, PSU, w_final,
            stop("'Dom' are duplicate column names: ", 
                  paste(names(Dom)[duplicated(names(Dom))], collapse = ","))
     if (nrow(Dom) != n) stop("'Dom' and 'Y' must be equal row count")
-    if (is.null(names(Dom))) stop("'Dom' must have column names")
     Dom[, (names(Dom)) := lapply(.SD, as.character)]
-    if (any(is.na(Dom))) stop("'Dom' has missing values")
+    if (anyNA(Dom)) stop("'Dom' has missing values")
     if (any(grepl("__", names(Dom)))) stop("'Dom' is not allowed column names with '__'")
   }
   
   namesZ <- NULL
   if (!is.null(Z)) {
     Z <- data.table(Z, check.names = TRUE)
+    if (anyNA(Z)) stop("'Z' has missing values")
     if (nrow(Z) != n) stop("'Z' and 'Y' must be equal row count")
     if (ncol(Z) != m) stop("'Z' and 'Y' must be equal column count")
-    if (any(is.na(Z))) stop("'Z' has missing values")
-    if (is.null(names(Z))) stop("'Z' must have column names")
     if (any(grepl("__", names(Z)))) stop("'Z' is not allowed column names with '__'")
     namesZ <- names(Z)
   }
   
-# countryX
+  # X
+  if (!is.null(X)) {
+      X <- data.table(X, check.names = TRUE)
+      if (!all(sapply(X, is.numeric))) stop("'X' must be numeric values")
+      if (anyNA(X)) stop("'X' has missing values")
+   }
+
+  # countryX
    if (!is.null(X)) {
      if(!is.null(countryX)) {
          countryX <- data.table(countryX)
          if (nrow(countryX) != nrow(X)) stop("'countryX' length must be equal with 'X' row count")
          if (ncol(countryX) != 1) stop("'countryX' has more than 1 column")
          countryX[, (names(countryX)) := lapply(.SD, as.character)]
-         if (any(is.na(countryX))) stop("'countryX' has missing values")
+         if (anyNA(countryX)) stop("'countryX' has missing values")
          if (names(countryX) != names(country)) stop("'countryX' must be equal with 'country' names")
          countrX <- countryX[, .N, keyby = names(countryX)][, N := NULL]
          countr <- country[, .N, keyby = names(country)][, N := NULL]
@@ -225,7 +228,7 @@ vardcrosannual <- function(Y, H, PSU, w_final,
       if (is.null(yearsX)) stop("'yearsX' must be defined")
       yearsX <- data.table(yearsX)
       yearsX[, (names(yearsX)) := lapply(.SD, as.character)]
-      if (any(is.na(yearsX))) stop("'yearsX' has missing values")
+      if (anyNA(yearsX)) stop("'yearsX' has missing values")
       if (any(duplicated(names(yearsX)))) 
                   stop("'yearsX' are duplicate column names: ", 
                        paste(names(yearsX)[duplicated(names(yearsX))], collapse = ","))
@@ -247,7 +250,7 @@ vardcrosannual <- function(Y, H, PSU, w_final,
       if (is.null(subperiodsX)) stop("'subperiodsX' must be defined")
       subperiodsX <- data.table(subperiodsX)
       subperiodsX[, (names(subperiodsX)) := lapply(.SD, as.character)]
-      if (any(is.na(subperiodsX))) stop("'subperiodsX' has missing values")
+      if (anyNA(subperiodsX)) stop("'subperiodsX' has missing values")
       if (any(duplicated(names(subperiodsX)))) 
                  stop("'subperiodsX' are duplicate column names: ", 
                           paste(names(subperiodsX)[duplicated(names(subperiodsX))], collapse = ","))
@@ -264,34 +267,33 @@ vardcrosannual <- function(Y, H, PSU, w_final,
       if (any(peri != periX) & !is.null(country)) stop("'unique(country, years, subperiods)' and 'unique(countryX, yearsX, subperiodsX)' records have different")
     } 
 
-    # X_ID_level1
-    if (!is.null(X)) {
-       X_ID_level1 <- data.table(X_ID_level1)
-       X_ID_level1[, (names(X_ID_level1)) := lapply(.SD, as.character)]
-       if (any(is.na(X_ID_level1))) stop("'X_ID_level1' has missing values")
-       if (nrow(X) != nrow(X_ID_level1)) stop("'X' and 'X_ID_level1' have different row count")
-       if (ncol(X_ID_level1) != 1) stop("'X_ID_level1' must be 1 column data.frame, matrix, data.table")
- 
-       ID_level1h <- copy(ID_level1)
-       if (!is.null(countryX)) {X_ID_level1 <- data.table(countryX, X_ID_level1)
-                                ID_level1h <- data.table(country, ID_level1h)}
-       X_ID_level1 <- data.table(yearsX, subperiodsX, X_ID_level1)
-       ID_level1h <- data.table(years, subperiods, ID_level1h)
-       ID_level1h <- ID_level1h[, .N, by = names(ID_level1h)][, N := NULL]
-       if (nrow(X_ID_level1[,.N, by = names(X_ID_level1)][N > 1]) > 0) stop("'X_ID_level1' have duplicates")
-       setkeyv(X_ID_level1, names(X_ID_level1))
-       setkeyv(ID_level1h, names(ID_level1h))
-       nperIDh <- names(ID_level1h)
-       if (any(nperIDh != names(X_ID_level1))) stop("'X_ID_level1' and 'ID_level1' must be equal names")
-       if (ID_level1h[, class(get(nperIDh))] != X_ID_level1[, class(get(nperIDh))])  stop("Class for 'X_ID_level1' and class for 'ID_level1' must be equal ")
+   # X_ID_level1
+   if (!is.null(X)) {
+      if (is.null(X_ID_level1)) stop("'X_ID_level1' must be defined")
+      X_ID_level1 <- data.table(X_ID_level1)
+      X_ID_level1[, (names(X_ID_level1)) := lapply(.SD, as.character)]
+      if (anyNA(X_ID_level1)) stop("'X_ID_level1' has missing values")
+      if (nrow(X) != nrow(X_ID_level1)) stop("'X' and 'X_ID_level1' have different row count")
+      if (any(names(X_ID_level1) != names(ID_level1))) stop("'X_ID_level1' and 'ID_level1' must be equal names")
+      if (ncol(X_ID_level1) != 1) stop("'X_ID_level1' must be 1 column data.frame, matrix, data.table")
 
-       if (!is.null(country)) {
-           if (nrow(ID_level1h) != nrow(X_ID_level1)) stop("'unique(countryX, periodX, X_ID_level1)' and 'unique(country, period, ID_level1)' have different row count")
-           if (any(ID_level1h != X_ID_level1)) stop("''unique(countryX, periodX, X_ID_level1)' and 'unique(country, period, ID_level1)' records have different")
-         } else {
-           if (nrow(ID_level1h) != nrow(X_ID_level1)) stop("'unique(periodX, X_ID_level1)' and 'unique(period, ID_level1)' have different row count")
-           if (any(ID_level1h != X_ID_level1)) stop("''unique(periodX, X_ID_level1)' and 'unique(period, ID_level1)' records have different")  }
-     }
+      ID_level1h <- data.table(years, subperiods, ID_level1)
+      X_ID_level1h <- data.table(yearsX, subperiodsX, X_ID_level1)
+      if (!is.null(countryX)) {X_ID_level1h <- data.table(countryX, X_ID_level1h)
+                               ID_level1h <- data.table(country, ID_level1h)}                      
+      ID_level1h <- ID_level1h[, .N, by = names(ID_level1h)][, N := NULL]
+      if (nrow(X_ID_level1h[,.N, by = names(X_ID_level1h)][N > 1]) > 0) stop("'X_ID_level1' have duplicates")  
+
+      setkeyv(X_ID_level1h, names(X_ID_level1h))
+      setkeyv(ID_level1h, names(ID_level1h))
+      if (!is.null(country)) {
+             if (nrow(ID_level1h) != nrow(X_ID_level1h)) stop("'unique(countryX, yearsX, subperiodsX, X_ID_level1)' and 'unique(country, years, subperiods, ID_level1)' have different row count")
+             if (any(ID_level1h != X_ID_level1h)) stop("''unique(countryX, periodX, X_ID_level1)' and 'unique(country, period, ID_level1)' records have different")
+           } else {
+             if (nrow(ID_level1h) != nrow(X_ID_level1h)) stop("'unique(yearsX, subperiodsX, X_ID_level1)' and 'unique(years, subperiods, ID_level1)' have different row count")
+             if (any(ID_level1h != X_ID_level1h)) stop("''unique(yearsX, subperiodsX, X_ID_level1)' and 'unique(years, subperiods, ID_level1)' records have different")  }       
+      ID_level1h <- X_ID_level1h <- NULL
+    }
 
    # ind_gr
    if (!is.null(X)) {
@@ -300,24 +302,9 @@ vardcrosannual <- function(Y, H, PSU, w_final,
        if (nrow(ind_gr) != nrow(X)) stop("'ind_gr' length must be equal with 'X' row count")
        if (ncol(ind_gr) != 1) stop("'ind_gr' must be 1 column data.frame, matrix, data.table")
        ind_gr[, (names(ind_gr)) := lapply(.SD, as.character)]
-       if (any(is.na(ind_gr))) stop("'ind_gr' has missing values")
+       if (anyNA(ind_gr)) stop("'ind_gr' has missing values")
     }
 
-   # X
-   if (!is.null(X)) {
-        X1 <- data.table(X, check.names = TRUE)
-        nX1 <- names(X1)
-        ind_gr1 <- copy(ind_gr) 
-        if (!is.null(periodX)) ind_gr1 <- data.table(periodX, ind_gr1, check.names = TRUE)
-        X2 <- data.table(ind_gr1, X1)
-        X1 <- X2[, .N, keyby = names(ind_gr1)][[ncol(ind_gr1) + 1]]
-        X2 <- X2[, lapply(.SD, function(x) sum(!is.na(x))), keyby = names(ind_gr1), .SDcols = nX1]
-        X2 <- X2[, !(names(X2) %in% names(ind_gr1)), with = FALSE]
-
-        if (!all(X2 == 0 | X1 == X2)) stop("X has missing values")
-        ind_gr1 <- nX1 <- nX2 <- X1 <- X2 <- NULL
-     }
-
    # g
    if (!is.null(X)) {
      if (is.null(class(g)) | all(class(g) == "function")) stop("'g' must be numeric")
@@ -326,7 +313,7 @@ vardcrosannual <- function(Y, H, PSU, w_final,
      if (ncol(g) != 1) stop("'g' must be 1 column data.frame, matrix, data.table")
      g <- g[, 1]
      if (!is.numeric(g)) stop("'g' must be numeric")
-     if (any(is.na(g))) stop("'g' has missing values")
+     if (anyNA(g)) stop("'g' has missing values")
      if (any(g == 0)) stop("'g' value can not be 0")
    }
     
@@ -339,7 +326,7 @@ vardcrosannual <- function(Y, H, PSU, w_final,
      if (ncol(q) != 1) stop("'q' must be 1 column data.frame, matrix, data.table")
      q <- q[, 1]
      if (!is.numeric(q)) stop("'q' must be numeric")
-     if (any(is.na(q))) stop("'q' has missing values")
+     if (anyNA(q)) stop("'q' has missing values")
      if (any(is.infinite(q))) stop("'q' value can not be infinite")
    }
 
@@ -348,10 +335,10 @@ vardcrosannual <- function(Y, H, PSU, w_final,
    var_est2 <- se  <- CI_lower <- CI_upper <- NULL
 
    pers <- data.table(years, subperiods, 
-                      pers=paste0(years[[names(years)]], "__", subperiods[[names(subperiods)]]))
+                      pers = paste0(years[[names(years)]], "__", subperiods[[names(subperiods)]]))
    if (!is.null(X)) persX <- data.table(yearsX, subperiodsX, 
                                   pers = paste0(yearsX[[names(yearsX)]], "__", subperiodsX[[names(subperiodsX)]]))
-   sarak <- pers[, .N, keyby = names(pers)][, N:=NULL]
+   sarak <- pers[, .N, keyby = names(pers)][, N := NULL]
    
    namesDom <- names(Dom)  
    year1 <- years[, .N, by = yearm][, N := NULL]
@@ -375,9 +362,9 @@ vardcrosannual <- function(Y, H, PSU, w_final,
                  datas <- vardchanges(Y = Y, H = H, PSU = PSU, w_final = w_final,
                                       ID_level1 = ID_level1, ID_level2 = ID_level2,
                                       Dom = Dom, Z = Z, country = country,
-                                      period = pers[, "pers", with = FALSE], dataset = NULL,
+                                      period = pers[, "pers"], dataset = NULL,
                                       period1 = yrs[["pers_1"]], period2 = yrs[["pers_2"]],
-                                      X = X, countryX = countryX, periodX = pers[, "pers", with = FALSE],
+                                      X = X, countryX = countryX, periodX = persX[, "pers"],
                                       X_ID_level1 = X_ID_level1, ind_gr = ind_gr,
                                       g = g, q = q, datasetX = NULL, annual = TRUE,
                                       linratio = !is.null(Z), percentratio = percentratio,
@@ -455,6 +442,10 @@ vardcrosannual <- function(Y, H, PSU, w_final,
                       rho1, A_matrix, annual_cros, ysum)             
    })
 
+  Y <- H <- PSU <- w_final <- ID_level1 <- ID_level1 <- ID_level2 <- NULL   
+  Dom <- Z <- country <- period <- X <- countryX <- countryX <- NULL
+  periodX <- X_ID_level1 <- ind_gr <- g <- q <- NULL
+
   crossectional_results <- rbindlist(lapply(apst, function(x) x[[1]]))
   crossectional_var_grad <- rbindlist(lapply(apst, function(x) x[[2]]))
   grad_var <- rbindlist(lapply(apst, function(x) x[[3]]))
