@@ -93,7 +93,7 @@ vardomh <- function(Y, H, PSU, w_final,
           if (min(q %in% names(datasetX)) == 1) q <- datasetX[, q, with = FALSE] } 
      }
 
-  equal_dataset <- identical(dataset, datasetX) & !is.null(X)
+  equal_dataset <- identical(dataset, datasetX) & !is.null(datasetX) & !is.null(X)
   if (equal_dataset) X_ID_level1 <- ID_level1
   N <- dataset <- datasetX <- NULL
 
@@ -102,7 +102,7 @@ vardomh <- function(Y, H, PSU, w_final,
   n <- nrow(Y)
   m <- ncol(Y)
   if (!all(sapply(Y, is.numeric))) stop("'Y' must be numeric values")
-  if (any(is.na(Y))) stop("'Y' has missing values")
+  if (anyNA(Y)) stop("'Y' has missing values")
   if (is.null(names(Y))) stop("'Y' must have column names")
   if (any(grepl("__", names(Y)))) stop("'Y' is not allowed column names with '__'")
   
@@ -110,9 +110,8 @@ vardomh <- function(Y, H, PSU, w_final,
   H <- data.table(H)
   if (nrow(H) != n) stop("'H' length must be equal with 'Y' row count")
   if (ncol(H) != 1) stop("'H' must be 1 column data.frame, matrix, data.table")
-  if (is.null(names(H))) stop("'H' must have column names")
   H[, (names(H)) := lapply(.SD, as.character)]
-  if (any(is.na(H))) stop("'H' has missing values")
+  if (anyNA(H)) stop("'H' has missing values")
   
   # PSU
   PSU <- data.table(PSU)
@@ -120,15 +119,15 @@ vardomh <- function(Y, H, PSU, w_final,
   if (nrow(PSU) != n) stop("'PSU' length must be equal with 'Y' row count")
   if (ncol(PSU) != 1) stop("'PSU' has more than 1 column")
   PSU[, (names(PSU)) := lapply(.SD, as.character)]
-  if (any(is.na(PSU))) stop("'PSU' has missing values")
+  if (anyNA(PSU)) stop("'PSU' has missing values")
   
   # w_final 
   w_final <- data.frame(w_final)
+  if (anyNA(w_final)) stop("'w_final' has missing values") 
   if (nrow(w_final) != n) stop("'w_final' must be equal with 'Y' row count")
   if (ncol(w_final) != 1) stop("'w_final' must be a vector or 1 column data.frame, matrix, data.table")
   w_final <- w_final[, 1]
   if (!is.numeric(w_final)) stop("'w_final' must be numeric")
-  if (any(is.na(w_final))) stop("'w_final' has missing values") 
 
   # period     
   if (!is.null(period)) {
@@ -138,7 +137,7 @@ vardomh <- function(Y, H, PSU, w_final,
                      paste(names(period)[duplicated(names(period))], collapse = ","))
       if (nrow(period) != n) stop("'period' must be the same length as 'Y'")
       period[, (names(period)) := lapply(.SD, as.character)]
-      if(any(is.na(period))) stop("'period' has missing values")
+      if(anyNA(period)) stop("'period' has missing values")
   } 
   np <- sum(ncol(period))
 
@@ -146,7 +145,7 @@ vardomh <- function(Y, H, PSU, w_final,
   if (is.null(ID_level2)) ID_level2 <- 1 : n
   ID_level2 <- data.table(ID_level2)
   ID_level2[, (names(ID_level2)) := lapply(.SD, as.character)]
-  if (any(is.na(ID_level2))) stop("'ID_level2' has missing values")
+  if (anyNA(ID_level2)) stop("'ID_level2' has missing values")
   if (ncol(ID_level2) != 1) stop("'ID_level2' must be 1 column data.frame, matrix, data.table")
   if (nrow(ID_level2) != n) stop("'ID_level2' length must be equal with 'Y' row count")
   if (is.null(period)){ if (any(duplicated(ID_level2))) stop("'ID_level2' are duplicate values") 
@@ -155,12 +154,11 @@ vardomh <- function(Y, H, PSU, w_final,
                                dd <- NULL}
 
   # ID_level1
-  if (is.null(ID_level1)) { stop("'ID_level1' takes 'ID_level2' values")
-                            ID_level1 <- copy(ID_level2) }
+  if (is.null(ID_level1)) stop("'ID_level1' must be defined")
   ID_level1 <- data.table(ID_level1)
   if (names(ID_level1) == names(PSU)) setnames(PSU, names(PSU), paste0(names(PSU), "_PSU"))
   ID_level1[, (names(ID_level1)) := lapply(.SD, as.character)]
-  if (any(is.na(ID_level1))) stop("'ID_level1' has missing values")
+  if (anyNA(ID_level1)) stop("'ID_level1' has missing values")
   if (ncol(ID_level1) != 1) stop("'ID_level1' must be 1 column data.frame, matrix, data.table")
   if (nrow(ID_level1) != n) stop("'ID_level1' must be the same length as 'Y'")
   if (names(ID_level2) == names(ID_level1)) setnames(ID_level2, names(ID_level2), paste0(names(ID_level2), "_id"))
@@ -169,14 +167,14 @@ vardomh <- function(Y, H, PSU, w_final,
   # PSU_sort
   if (!is.null(PSU_sort)) {
           PSU_sort <- data.frame(PSU_sort)
+          if (anyNA(PSU_sort)) stop("'PSU_sort' has missing values")
           if (nrow(PSU_sort) != n) stop("'PSU_sort' must be equal with 'Y' row count")
           if (ncol(PSU_sort) != 1) stop("'PSU_sort' must be a vector or 1 column data.frame, matrix, data.table")
           PSU_sort <- PSU_sort[, 1]
           if (!is.numeric(PSU_sort)) stop("'PSU_sort' must be numeric")
-          if (any(is.na(PSU_sort))) stop("'PSU_sort' has missing values")
 
           psuag <- data.table(PSU, PSU_sort)
-          if (!is.null(period)) hpY <- data.table(period, psuag)
+          if (!is.null(period)) psuag <- data.table(period, psuag)
           psuag <- psuag[,.N, by = names(psuag)][,N := NULL]
           psuag <- psuag[,.N, by = c(names(period), names(PSU))]
           if (nrow(psuag[N > 1]) > 0) stop("'PSU_sort' must be equal for each 'PSU'")
@@ -186,25 +184,22 @@ vardomh <- function(Y, H, PSU, w_final,
   # N_h
   if (!is.null(N_h)) {
       N_h <- data.table(N_h)
+      if (anyNA(N_h)) stop("'N_h' has missing values")
       if (ncol(N_h) != np + 2) stop(paste0("'N_h' should be ", np + 2, " columns"))
       if (!is.numeric(N_h[[ncol(N_h)]])) stop("The last column of 'N_h' should be numeric")
-      if (any(is.na(N_h))) stop("'N_h' has missing values") 
-      if (is.null(names(N_h))) stop("'N_h' must have column names")
-      if (all(names(H) %in% names(N_h))) {N_h[, (names(H)) := as.character(get(names(H)))]
-             } else stop("All strata titles of 'H' have not in 'N_h'")
+      nams <- c(names(period), names(H))
+      if (all(nams %in% names(N_h))) {N_h[, (nams) := lapply(.SD, as.character), .SDcols = nams]
+             } else stop(paste0("All strata titles of 'H'", ifelse(!is.null(period), "and periods titles of 'period'", ""), " have not in 'N_h'"))
       if (is.null(period)) {
-             if (names(H) != names(N_h)[1]) stop("Strata titles for 'H' and 'N_h' is not equal")
              if (any(is.na(merge(unique(H), N_h, by = names(H), all.x = TRUE)))) stop("'N_h' is not defined for all strata")
              if (any(duplicated(N_h[, head(names(N_h), -1), with = FALSE]))) stop("Strata values for 'N_h' must be unique")
        } else { pH <- data.table(period, H)
-                if (any(names(pH) != names(N_h)[c(1 : (1 + np))])) stop("Strata titles for 'period' with 'H' and 'N_h' is not equal")
-                nperH <- names(period)
-                N_h[, (nperH) := as.character(get(nperH))]
                 if (any(is.na(merge(unique(pH), N_h, by = names(pH), all.x = TRUE)))) stop("'N_h' is not defined for all strata and periods")
                 if (any(duplicated(N_h[, head(names(N_h), -1), with = FALSE]))) stop("Strata values for 'N_h' must be unique in all periods")
                 pH <- NULL
-     }
-    setkeyv(N_h, names(N_h)[c(1 : (1 + np))])
+              }
+
+      setkeyv(N_h, names(N_h)[c(1 : (1 + np))])
   }
 
   # Dom
@@ -214,10 +209,9 @@ vardomh <- function(Y, H, PSU, w_final,
     if (any(duplicated(names(Dom)))) 
           stop("'Dom' are duplicate column names: ", 
                paste(names(Dom)[duplicated(names(Dom))], collapse = ","))
-    if (nrow(Dom) != n) stop("'Dom' and 'Y' must be equal row count")
-    if (is.null(names(Dom))) stop("'Dom' must have column names")
     Dom[, (names(Dom)) := lapply(.SD, as.character)]
-    if (any(is.na(Dom))) stop("'Dom' has missing values")
+    if (anyNA(Dom)) stop("'Dom' has missing values")
+    if (nrow(Dom) != n) stop("'Dom' and 'Y' must be equal row count")
     if (any(grepl("__", names(Dom)))) stop("'Dom' is not allowed column names with '__'")
     namesDom <- names(Dom)
   }
@@ -225,17 +219,18 @@ vardomh <- function(Y, H, PSU, w_final,
   # Z
   if (!is.null(Z)) {
     Z <- data.table(Z, check.names = TRUE)
+    if (anyNA(Z)) stop("'Z' has missing values")
     if (nrow(Z) != n) stop("'Z' and 'Y' must be equal row count")
     if (ncol(Z) != m) stop("'Z' and 'Y' must be equal column count")
     if (!all(sapply(Z, is.numeric))) stop("'Z' must be numeric values")
-    if (any(is.na(Z))) stop("'Z' has missing values")
-    if (is.null(names(Z))) stop("'Z' must have column names")
     if (any(grepl("__", names(Z)))) stop("'Z' is not allowed column names with '__'")
   }
  
+  # X
   if (!is.null(X)) {
     X <- data.table(X, check.names = TRUE)
     if (!all(sapply(X, is.numeric))) stop("'X' must be numeric values")
+    if (anyNA(X)) stop("'X' has missing values")
   }
 
   # periodX
@@ -243,7 +238,7 @@ vardomh <- function(Y, H, PSU, w_final,
      if(!is.null(periodX)) {
         periodX <- data.table(periodX)
         periodX[, (names(periodX)) := lapply(.SD, as.character)]
-        if (any(is.na(periodX))) stop("'periodX' has missing values")
+        if (anyNA(periodX)) stop("'periodX' has missing values")
         if (any(duplicated(names(periodX)))) 
                     stop("'periodX' are duplicate column names: ", 
                          paste(names(periodX)[duplicated(names(periodX))], collapse = ","))
@@ -261,20 +256,17 @@ vardomh <- function(Y, H, PSU, w_final,
     if (is.null(X_ID_level1)) stop("'X_ID_level1' must be defined")
     X_ID_level1 <- data.table(X_ID_level1)
     X_ID_level1[, (names(X_ID_level1)) := lapply(.SD, as.character)]
-    if (any(is.na(X_ID_level1))) stop("'X_ID_level1' has missing values")
+    if (anyNA(X_ID_level1)) stop("'X_ID_level1' has missing values")
     if (nrow(X) != nrow(X_ID_level1)) stop("'X' and 'X_ID_level1' have different row count")
     if (ncol(X_ID_level1) != 1) stop("'X_ID_level1' must be 1 column data.frame, matrix, data.table")
+    if (any(names(X_ID_level1) != names(ID_level1))) stop("'X_ID_level1' and 'ID_level1' must be equal names")
 
-    ID_level1h <- data.table(unique(ID_level1))
+    ID_level1h <- copy(ID_level1)
     X_ID_level1h <- copy(X_ID_level1)
     if (!is.null(periodX)) {X_ID_level1h <- data.table(periodX, X_ID_level1h)
-                           ID_level1h <- data.table(period, ID_level1)
-                           ID_level1h <- ID_level1h[, .N, by = names(ID_level1h)][, N := NULL]}
- 
+                           ID_level1h <- data.table(period, ID_level1)}                       
+    ID_level1h <- ID_level1h[, .N, by = names(ID_level1h)][, N := NULL]
     if (nrow(X_ID_level1h[,.N, by = names(X_ID_level1h)][N > 1]) > 0) stop("'X_ID_level1' have duplicates")
-    nperIDh <- names(ID_level1h)
-    if (any(nperIDh != names(X_ID_level1h))) stop("'X_ID_level1' and 'ID_level1' must be equal names")
-    if (ID_level1h[, class(get(nperIDh))] != X_ID_level1h[, class(get(nperIDh))])  stop("Class for 'X_ID_level1' and class for 'ID_level1' must be equal ")
 
     setkeyv(X_ID_level1h, names(X_ID_level1h))
     setkeyv(ID_level1h, names(ID_level1h))
@@ -285,10 +277,9 @@ vardomh <- function(Y, H, PSU, w_final,
         if (nrow(ID_level1h) != nrow(X_ID_level1h)) stop("'X_ID_level1' and 'unique(ID_level1)' have different row count")
         if (any(ID_level1h != X_ID_level1h)) stop("'X_ID_level1' and 'unique(ID_level1)' records have different")
      }
-    X_ID_level1h <- NULL
+    ID_level1h <- X_ID_level1h <- NULL
   }
 
-
   # ind_gr
   if (!is.null(X)) {
      if(is.null(ind_gr)) ind_gr <- rep("1", nrow(X)) 
@@ -296,24 +287,9 @@ vardomh <- function(Y, H, PSU, w_final,
      if (nrow(ind_gr) != nrow(X)) stop("'ind_gr' length must be equal with 'X' row count")
      if (ncol(ind_gr) != 1) stop("'ind_gr' must be 1 column data.frame, matrix, data.table")
      ind_gr[, (names(ind_gr)) := lapply(.SD, as.character)]
-     if (any(is.na(ind_gr))) stop("'ind_gr' has missing values")
+     if (anyNA(ind_gr)) stop("'ind_gr' has missing values")
    }
 
-  # X
-  if (!is.null(X)) {
-       X1 <- data.table(X, check.names = TRUE)
-       nX1 <- names(X1)
-       ind_gr1 <- copy(ind_gr) 
-       if (!is.null(periodX)) ind_gr1 <- data.table(periodX, ind_gr1, check.names = TRUE)
-       X2 <- data.table(ind_gr1, X1)
-       X1 <- X2[, .N, keyby = names(ind_gr1)][[ncol(ind_gr1) + 1]]
-       X2 <- X2[, lapply(.SD, function(x) sum(!is.na(x))), keyby = names(ind_gr1), .SDcols = nX1]
-       X2 <- X2[, !(names(X2) %in% names(ind_gr1)), with = FALSE]
-
-       if (!all(X2 == 0 | X1 == X2)) stop("X has missing values")
-       ind_gr1 <- nX1 <- nX2 <- X1 <- X2 <- NULL
-    }
-
   # g
   if (!is.null(X)) {
     if (is.null(class(g)) | all(class(g) == "function")) stop("'g' must be numeric")
@@ -322,7 +298,7 @@ vardomh <- function(Y, H, PSU, w_final,
     if (ncol(g) != 1) stop("'g' must be 1 column data.frame, matrix, data.table")
     g <- g[, 1]
     if (!is.numeric(g)) stop("'g' must be numeric")
-    if (any(is.na(g))) stop("'g' has missing values")
+    if (anyNA(g)) stop("'g' has missing values")
     if (any(g == 0)) stop("'g' value can not be 0")
    }
     
@@ -335,7 +311,7 @@ vardomh <- function(Y, H, PSU, w_final,
     if (ncol(q) != 1) stop("'q' must be 1 column data.frame, matrix, data.table")
     q <- q[, 1]
     if (!is.numeric(q)) stop("'q' must be numeric")
-    if (any(is.na(q))) stop("'q' has missing values")
+    if (anyNA(q)) stop("'q' has missing values")
     if (any(is.infinite(q))) stop("'q' value can not be infinite")
   }
   
@@ -343,6 +319,7 @@ vardomh <- function(Y, H, PSU, w_final,
   # Domains
 
   if (!is.null(Dom)) Y1 <- domain(Y, Dom) else Y1 <- Y
+  Y <- NULL
   n_nonzero <- copy(Y1)
   if (!is.null(period)){ n_nonzero <- data.table(period, n_nonzero) 
                          n_nonzero <- n_nonzero[, lapply(.SD, function(x) 
@@ -410,19 +387,53 @@ vardomh <- function(Y, H, PSU, w_final,
     } else {
             Y2 <- Y1
           }
-  lin1 <- lin2 <- NULL
+
+
+  # Total estimation
+  lin1 <- Y_est <- Z_est <- .SD <- variableDZ <- NULL
+
+  hY <- data.table(Y1 * w_final)
+  if (is.null(period)) { Y_est <- hY[, lapply(.SD, sum, na.rm = TRUE), .SDcols = names(Y1)]
+                } else { hY <- data.table(period0, hY)
+                         Y_est <- hY[, lapply(.SD, sum, na.rm = TRUE), keyby = names(period), .SDcols = names(Y1)]
+                       }
+  Y_est <- transpos(Y_est, is.null(period), "Y_est", names(period))
+  all_result <- Y_est
+  
+  if (!is.null(Z1)) {
+         YZnames <- data.table(variable = names(Y1), variableDZ = names(Z1))
+         setkeyv(YZnames, "variable")
+         setkeyv(all_result, "variable")
+         all_result <- merge(all_result, YZnames)
+         
+         hZ <- data.table(Z1 * w_final)
+         if (is.null(period)) { Z_est <- hZ[, lapply(.SD, sum, na.rm = TRUE), .SDcols = names(Z1)]
+                       } else { hZ <- data.table(period, hZ)
+                                Z_est <- hZ[, lapply(.SD, sum, na.rm = TRUE), keyby = names(period), .SDcols = names(Z1)]
+                              }
+         Z_est <- transpos(Z_est, is.null(period), "Z_est", names(period), "variableDZ")
+         all_result <- merge(all_result, Z_est, all = TRUE, by = c(names(period), "variableDZ"))                                            
+      }
+
+  vars <- data.table(variable = names(Y1), nr_names = 1 : ncol(Y1))
+  all_result <- merge(vars, all_result, by = "variable")
+
+  n_nonzero <- transpos(n_nonzero, is.null(period), "n_nonzero", names(period))
+  all_result <- merge(all_result, n_nonzero, all = TRUE, by = c(names(period), "variable"))
+  n_nonzero <- vars <- Y1 <- Z1 <- Y_est <- Z_est <- hY <- hZ <- YZnames <- NULL
+
 
   # Calibration
 
-  YY <- data.table(idper, ID_level1, H, PSU)
-  if (!is.null(PSU_sort)) YY <- data.table(YY, PSU_sort)
-  YY <- data.table(YY, w_design, w_final, Y2)
+  YY <- data.table(idper, ID_level1, H, PSU, check.names = TRUE)
+  if (!is.null(PSU_sort)) YY <- data.table(YY, PSU_sort, check.names = TRUE)
+  YY <- data.table(YY, w_design, w_final, Y2, check.names = TRUE)
 
   YY2 <- YY[, lapply(.SD, sum, na.rm = TRUE), by = c(names(YY)[c(2 : (6 + np + psusn))]),
                                              .SDcols = names(YY)[-(1 : (6 + np + psusn))]]
   Y3 <- YY2[, c(-(1 : (5 + np + psusn))), with = FALSE]
 
-  period <- NULL
+  idper <- period <- NULL
   if (np > 0) period <- YY2[, c(1 : np), with = FALSE]
 
   ID_level1h <- YY2[, np + 1, with = FALSE]
@@ -441,17 +452,17 @@ vardomh <- function(Y, H, PSU, w_final,
   # Calibration
   res_outp <- NULL
   if (!is.null(X)) {
-       if (np>0) ID_level1h <- data.table(period, ID_level1h)
+       if (np > 0) ID_level1h <- data.table(period, ID_level1h)
        setnames(ID_level1h, names(ID_level1h), names(X_ID_level1))
        X0 <- data.table(X_ID_level1, ind_gr, q, g, X)
        D1 <- merge(ID_level1h, X0, by = names(ID_level1h), sort = FALSE)
 
        ind_gr <- D1[, np + 2, with = FALSE]
        if (!is.null(period)) ind_gr <- data.table(D1[, names(periodX), with = FALSE], ind_gr)
-       ind_period <- do.call("paste", c(as.list(ind_gr), sep="_"))
+       ind_period <- do.call("paste", c(as.list(ind_gr), sep = "_"))
     
        lin1 <- lapply(split(Y3[, .I], ind_period), function(i) 
-                 data.table(sar_nr=i, 
+                 data.table(sar_nr = i, 
                             residual_est(Y = Y3[i],
                                          X = D1[i, (np + 5) : ncol(D1), with = FALSE],
                                          weight = w_design2[i],
@@ -461,7 +472,8 @@ vardomh <- function(Y, H, PSU, w_final,
        Y4[, sar_nr := NULL]
        if (outp_res) res_outp <- data.table(ID_level1h, PSU, w_final2, Y4)
    } else Y4 <- Y3
-                               
+  X0 <- D1 <- X_ID_level1 <- ID_level1h <- ind_gr <- lin1 <- X <- g <- q <- NULL                             
+
   var_est <- variance_est(Y = Y4, H = H, PSU = PSU,
                           w_final = w_final2,
                           N_h = N_h, fh_zero = fh_zero, 
@@ -470,10 +482,7 @@ vardomh <- function(Y, H, PSU, w_final,
                           period = period, dataset = NULL,
                           msg = "Current variance estimation")
   var_est <- transpos(var_est, is.null(period), "var_est", names(period))
-  all_result <- var_est
-
-  n_nonzero <- transpos(n_nonzero, is.null(period), "n_nonzero", names(period))
-  all_result <- merge(all_result, n_nonzero, all=TRUE)
+  all_result <- merge(all_result, var_est, all = TRUE, by = c(names(period), "variable"))
  
   # Variance of HT estimator under current design
   var_cur_HT <- variance_est(Y = Y3, H = H, PSU = PSU,
@@ -484,9 +493,9 @@ vardomh <- function(Y, H, PSU, w_final,
                              period = period, dataset = NULL,
                              msg = "Variance of HT estimator under current design")
   var_cur_HT <- transpos(var_cur_HT, is.null(period), "var_cur_HT", names(period))
-  all_result <- merge(all_result, var_cur_HT)
+  all_result <- merge(all_result, var_cur_HT, all = TRUE, by = c(names(period), "variable"))
   n_nonzero <- var_est <- var_cur_HT <- NULL
-  H <- PSU <- N_h <- NULL
+  H <- PSU <- PSU_sort <- N_h <- NULL
 
   # Variance of HT estimator under SRS
   if (is.null(period)) {
@@ -510,11 +519,12 @@ vardomh <- function(Y, H, PSU, w_final,
            S2_y_ca <- rbindlist(lapply(lin1, function(x) x[[3]]))
       }
   var_srs_HT <- transpos(var_srs_HT, is.null(period), "var_srs_HT", names(period))
-  all_result <- merge(all_result, var_srs_HT, all = TRUE)
+  all_result <- merge(all_result, var_srs_HT, all = TRUE, by = c(names(period), "variable"))
   S2_y_HT <- transpos(S2_y_HT, is.null(period), "S2_y_HT", names(period))
-  all_result <- merge(all_result, S2_y_HT, all = TRUE)
+  all_result <- merge(all_result, S2_y_HT, all = TRUE, by = c(names(period), "variable"))
   S2_y_ca <- transpos(S2_y_ca, is.null(period), "S2_y_ca", names(period))
-  all_result <- merge(all_result, S2_y_ca, all = TRUE)
+  all_result <- merge(all_result, S2_y_ca, all = TRUE, by = c(names(period), "variable"))
+  Y3 <- w_design2 <- var_srs_HT <- S2_y_HT <- S2_y_ca <- NULL
 
   # Variance of calibrated estimator under SRS
   if (is.null(period)) {
@@ -534,45 +544,11 @@ vardomh <- function(Y, H, PSU, w_final,
            var_srs_ca <- rbindlist(lapply(lin1, function(x) x[[2]]))
         }
   var_srs_ca <- transpos(var_srs_ca, is.null(period), "var_srs_ca", names(period), "variable")
-  all_result <- merge(all_result, var_srs_ca, all = TRUE)
+  all_result <- merge(all_result, var_srs_ca, all = TRUE, by = c(names(period), "variable"))
   S2_res <- transpos(S2_res, is.null(period), "S2_res", names(period), "variable")
-  all_result <- merge(all_result, S2_res, all = TRUE)
-  Y3 <- S2_y_HT <- S2_y_ca <- S2_res <- var_srs_HT <- NULL
-  var_srs_ca <- Y4 <- NULL
+  all_result <- merge(all_result, S2_res, all = TRUE, by = c(names(period), "variable"))
+  Y4 <- w_final2 <- var_srs_ca <- S2_res <- NULL
 
-  # Total estimation
-  Y_est <- Z_est <- .SD <- NULL
-
-  hY <- data.table(Y1 * w_final)
-  if (is.null(period)) { Y_est <- hY[, lapply(.SD, sum, na.rm = TRUE), .SDcols = names(Y1)]
-                } else { hY <- data.table(period0, hY)
-                         Y_est <- hY[, lapply(.SD, sum, na.rm = TRUE), keyby = names(period), .SDcols = names(Y1)]
-                       }
-  Y_est <- transpos(Y_est, is.null(period), "Y_est", names(period))
-  all_result <- merge(all_result, Y_est)
-  
-  if (!is.null(Z1)) {
-         YZnames <- data.table(variable = names(Y1), variableDZ = names(Z1))
-         setkeyv(YZnames, "variable")
-         setkeyv(all_result, "variable")
-         all_result <- merge(all_result, YZnames)
-         
-         hZ <- data.table(Z1 * w_final)
-         if (is.null(period)) { Z_est <- hZ[, lapply(.SD, sum, na.rm = TRUE), .SDcols = names(Z1)]
-                       } else { hZ <- data.table(period0, hZ)
-                                Z_est <- hZ[, lapply(.SD, sum, na.rm = TRUE), keyby = names(period), .SDcols = names(Z1)]
-                              }
-         Z_est <- transpos(Z_est, is.null(period), "Z_est", names(period), "variableDZ")
-         setkeyv(all_result, c(names(period), "variableDZ"))
-         all_result <- merge(all_result, Z_est, all = TRUE)                                            
-      }
-
-  vars <- data.table(variable = names(Y1), nr_names = 1 : ncol(Y1))
-  all_result <- merge(vars, all_result, by = "variable")
-                        
-  vars <- idper <- Y1 <- Z1 <- Y_est <- NULL
-  Z_est <- period0 <- hY <- hZ <- NULL
-  YZnames <- dati <- NULL
 
   all_result[, estim := Y_est]   
   if (!is.null(all_result$Z_est)) all_result[, estim := Y_est / Z_est * percentratio]
@@ -602,17 +578,17 @@ vardomh <- function(Y, H, PSU, w_final,
   all_result[, CI_lower := estim - tsad * se]
   all_result[, CI_upper := estim + tsad * se]
 
+  variableD <- NULL
   setnames(all_result, c("variable", "var_est"), c("variableD", "var"))
   if (!is.null(all_result$Z_est)) {
-                         nosrZ <- all_result$variableDZ
-                         nosrZ <- nosrZ[!duplicated(nosrZ)]
-                         nosrZ1 <- data.table(variableZ = t(data.frame(strsplit(nosrZ, "__")))[, c(1)])
-                         nosrZ <- data.table(variableDZ = nosrZ, nosrZ1)
-                         all_result <- merge(all_result, nosrZ, by="variableDZ")
-                         nosrZ <- nosrZ1 <- NULL
-                      }
+                       nosrZ <- data.table(all_result[, "variableDZ"], all_result[, tstrsplit(variableDZ, "__")][, 1])
+                       nosrZ <- nosrZ[!duplicated(nosrZ)]
+                       setnames(nosrZ, "V1", "variableZ")
+                       all_result <- merge(all_result, nosrZ, by = "variableDZ")
+                       nosrZ <- NULL
+                    }
 
-  nosr <- data.table(variableD = all_result$variableD, t(data.frame(strsplit(all_result$variableD, "__"))))
+  nosr <- data.table(all_result[, "variableD"], all_result[, tstrsplit(variableD, "__")])
   nosr <- nosr[!duplicated(nosr)]
   nosr <- nosr[, lapply(nosr, as.character)]
   setnames(nosr, names(nosr)[2], "variable")
