@@ -7,7 +7,6 @@ variance_othstr <- function(Y, H, H2, w_final, N_h = NULL, N_h2, period = NULL,
       if (min(Y %in% names(dataset)) != 1) stop("'Y' does not exist in 'dataset'!")
       if (min(Y %in% names(dataset)) == 1) Y <- dataset[, Y, with = FALSE] 
 
-
       if(!is.null(H)) {
           if (min(H %in% names(dataset)) != 1) stop("'H' does not exist in 'dataset'!")
           if (min(H %in% names(dataset)) == 1) H <- dataset[, H, with = FALSE] }
@@ -15,7 +14,6 @@ variance_othstr <- function(Y, H, H2, w_final, N_h = NULL, N_h2, period = NULL,
           if (min(H2 %in% names(dataset)) != 1) stop("'H2' does not exist in 'dataset'!")
           if (min(H2 %in% names(dataset)) == 1) H2 <- dataset[, H2, with = FALSE] }
 
-
       if(!is.null(w_final)) {
           if (min(w_final %in% names(dataset)) != 1) stop("'w_final' does not exist in 'dataset'!")
           if (min(w_final %in% names(dataset)) == 1) w_final <- dataset[, w_final, with = FALSE] }
@@ -31,25 +29,22 @@ variance_othstr <- function(Y, H, H2, w_final, N_h = NULL, N_h2, period = NULL,
   Y <- data.table(Y, check.names = TRUE)
   n <- nrow(Y)
   m <- ncol(Y)
-  if (!all(sapply(Y, is.numeric))) stop("'Y' must be numeric values")
-  if (any(is.na(Y))) print("'Y' has missing values")
-  if (is.null(names(Y))) stop("'Y' must have column names")
-  
+  if (anyNA(Y)) print("'Y' has missing values")
+  if (!all(sapply(Y, is.numeric))) stop("'Y' must be numeric values")  
+
   # H
   H <- data.table(H)
   if (nrow(H) != n) stop("'H' length must be equal with 'Y' row count")
   if (ncol(H) != 1) stop("'H' must be 1 column data.frame, matrix, data.table")
-  if (is.null(names(H))) stop("'H' must have column names")
   H[, (names(H)) := lapply(.SD, as.character)]
-  if (any(is.na(H))) stop("'H' has missing values")
+  if (anyNA(H)) stop("'H' has missing values")
 
   # H2
   H2 <- data.table(H2)
   if (nrow(H2) != n) stop("'H2' length must be equal with 'Y' row count")
   if (ncol(H2) != 1) stop("'H2' must be 1 column data.frame, matrix, data.table")
-  if (is.null(names(H2))) stop("'H2' must have column names")
   H2[, (names(H2)) := lapply(.SD, as.character)]
-  if (any(is.na(H2))) stop("'H2' has missing values")
+  if (anyNA(H2)) stop("'H2' has missing values")
 
   # w_final
   w_final <- data.frame(w_final)
@@ -57,7 +52,7 @@ variance_othstr <- function(Y, H, H2, w_final, N_h = NULL, N_h2, period = NULL,
   if (ncol(w_final) != 1) stop("'w_final' must be vector or 1 column data.frame, matrix, data.table")
   w_final <- w_final[, 1]
   if (!is.numeric(w_final)) stop("'w_final' must be numeric")
-  if (any(is.na(w_final))) stop("'w_final' has missing values")
+  if (anyNA(w_final)) stop("'w_final' has missing values")
 
   # period     
   if (!is.null(period)) {
@@ -67,29 +62,30 @@ variance_othstr <- function(Y, H, H2, w_final, N_h = NULL, N_h2, period = NULL,
                       paste(names(period)[duplicated(names(period))], collapse = ","))
        if (nrow(period) != n) stop("'period' must be the same length as 'Y'")
        period[, (names(period)) := lapply(.SD, as.character)]
-       if(any(is.na(period))) stop("'period' has missing values") 
+       if(anyNA(period)) stop("'period' has missing values") 
   }   
   np <- sum(ncol(period))
   
   # N_h
   if (!is.null(N_h)) {
       N_h <- data.table(N_h)
+      if (anyNA(N_h)) stop("'N_h' has missing values")
       if (ncol(N_h) != np + 2) stop(paste0("'N_h' should be ", np + 2, " columns"))
       if (!is.numeric(N_h[[ncol(N_h)]])) stop("The last column of 'N_h' should be numerical")
-      if (any(is.na(N_h))) stop("'N_h' has missing values") 
-      if (is.null(names(N_h))) stop("'N_h' must be colnames")
-      if (all(names(H) %in% names(N_h))) {N_h[, (names(H)) := lapply(.SD, as.character), .SDcols = names(H)]
-             } else stop("All strata titles of 'H' have not in 'N_h'")
+       
+      nams <- c(names(period), names(H))
+      if (all(nams %in% names(N_h))) {N_h[, (nams) := lapply(.SD, as.character), .SDcols = nams]
+             } else stop(paste0("All strata titles of 'H'", ifelse(!is.null(period), "and periods titles of 'period'", ""), " have not in 'N_h'"))
+   
       if (is.null(period)) {
-             if (names(H) != names(N_h)[1]) stop("Strata titles for 'H' and 'N_h' is not equal")
-             if (any(is.na(merge(unique(H), N_h, by = names(H), all.x = TRUE)))) stop("'N_h' is not defined for all stratas")
+             if (any(is.na(merge(unique(H), N_h, by = names(H), all.x = TRUE)))) stop("'N_h' is not defined for all strata")
+             if (any(duplicated(N_h[, head(names(N_h), -1), with = FALSE]))) stop("Strata values for 'N_h' must be unique")
        } else { pH <- data.table(period, H)
-                if (any(names(pH) != names(N_h)[c(1 : (1 + np))])) stop("Strata titles for 'period' with 'H' and 'N_h' is not equal")
-                nperH <- names(period)
-                N_h[, (nperH) := as.character(get(nperH))]
-                if (any(is.na(merge(unique(pH), N_h, by = names(pH), all.x = TRUE)))) stop("'N_h' is not defined for all stratas and periods")
-               } 
-     setkeyv(N_h, names(N_h)[c(1 : (1 + np))])
+                if (any(is.na(merge(unique(pH), N_h, by = names(pH), all.x = TRUE)))) stop("'N_h' is not defined for all strata and periods")
+                if (any(duplicated(N_h[, head(names(N_h), -1), with = FALSE]))) stop("Strata values for 'N_h' must be unique in all periods")
+                pH <- NULL
+              }
+      setkeyv(N_h, names(N_h)[c(1 : (1 + np))])
   } else {
     Nh <- data.table(H, w_final)
     if (!is.null(period)) Nh <- data.table(period, Nh)
@@ -101,19 +97,19 @@ variance_othstr <- function(Y, H, H2, w_final, N_h = NULL, N_h2, period = NULL,
   # N_h2
   if (!is.null(N_h2)) {
       N_h2 <- data.table(N_h2)
+      if (anyNA(N_h2)) stop("'N_h2' has missing values") 
       if (ncol(N_h2) != np + 2) stop(paste0("'N_h2' should be ", np + 2, " columns"))
       if (!is.numeric(N_h2[[ncol(N_h2)]])) stop("The last column of 'N_h2' should be numerical")
-      if (any(is.na(N_h2))) stop("'N_h2' has missing values") 
-      if (is.null(names(N_h2))) stop("'N_h2' must be colnames")
-      if (all(names(H2) %in% names(N_h2))) {N_h2[, (names(H2)) := lapply(.SD, as.character), .SDcols = names(H2)]
-             } else stop("All strata titles of 'H2' have not in 'N_h2'")
+
+      nams2 <- c(names(period), names(H2))
+      if (all(nams2 %in% names(N_h2))) {N_h2[, (nams2) := lapply(.SD, as.character), .SDcols = nams2]
+             } else stop(paste0("All strata titles of 'H2'", ifelse(!is.null(period), "and periods titles of 'period'", ""), " have not in 'N_h2'"))
+   
       if (is.null(period)) {
              if (names(H2) != names(N_h2)[1]) stop("Strata titles for 'H2' and 'N_h2' is not equal")
              if (any(is.na(merge(unique(H2), N_h2, by = names(H2), all.x = TRUE)))) stop("'N_h2' is not defined for all stratas")
        } else { pH2 <- data.table(period, H2)
                 if (any(names(pH2) != names(N_h2)[c(1 : (1 + np))])) stop("Strata titles for 'period' with 'H2' and 'N_h2' is not equal")
-                nperH <- names(period)
-                N_h2[, (nperH) := as.character(get(nperH))]
                 if (any(is.na(merge(unique(pH2), N_h2, by = names(pH2), all.x = TRUE)))) stop("'N_h2' is not defined for all stratas and periods")
                 } 
     setkeyv(N_h2, names(N_h2)[c(1 : (1 + np))])
@@ -136,11 +132,9 @@ variance_othstr <- function(Y, H, H2, w_final, N_h = NULL, N_h2, period = NULL,
   Ys[, paste0(names(Y),"_sc") := lapply(Y, function(x) x ^ 2)]
   Ys[, paste0(names(Y),"_sd") := Y]
 
-
   Ys <- data.table(H, H2, Ys)
   if (!is.null(period)) Ys <- data.table(period, Ys)
 
-
   # n_h1
   n_h1 <- data.table(H)
   if (!is.null(period))   n_h1 <- data.table(period, n_h1)
@@ -196,18 +190,15 @@ variance_othstr <- function(Y, H, H2, w_final, N_h = NULL, N_h2, period = NULL,
            paste0(names(Y), "_sd"), with = FALSE], function(x) 
                  (1 / n_h1) * x ^ 2  * pop ^ 2 * (1 / n_h1 - 1 / pop)/(n_h1 - 1))]
 
-
   z_h_h2[n_h1 == 1, paste0(names(Y), "_sc") := NA]
   z_h_h2[n_h1 == 1, paste0(names(Y), "_sd") := NA]
 
-
   nameszh2 <- names(H2)
   if (!is.null(period)) nameszh2 <- c(names(period), nameszh2)
   
   zh2 <- z_h_h2[, lapply(.SD, sum, na.rm = TRUE), keyby = nameszh2,
                       .SDcols = names(z_h_h2)[-(1 : (2 + np))]] 
 
-
   zh2 <- merge(zh2, F_h2, by = nn_h2)
   pop2 <- zh2[[names(N_h2)[ncol(N_h2)]]]
   nh2 <- zh2[["n_h2"]]
